/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ApplicationFrameDescriptor.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.guibuilder;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JFileChooser;

import org.ow2.jonas.autostart.builder.Builder;
import org.ow2.jonas.autostart.frame.ApplicationFrame;
import org.ow2.jonas.autostart.fullwizard.FullWizardPanelDescriptor;
import org.ow2.jonas.autostart.utility.CustomFileFilter;
import org.ow2.jonas.autostart.utility.Utility;

/**
 * This class creates main panel with application fields.
 * @author skabore
 */
public class ApplicationFrameDescriptor extends FullWizardPanelDescriptor implements ActionListener {

    /**
     * The identifier.
     */
    public static final String IDENTIFIER = "APPLICATION_CHOOSE_PANEL";

    /**
     * The application frame.
     */
    private ApplicationFrame applicationFrame;

    /**
     * The builder librairy.
     */
    private Builder builder;

    /**
     * The constructeur.
     * @param builder the builder librairy
     * @param steps, the required steps to create a starter
     * @param stepId, the step number
     */
    public ApplicationFrameDescriptor(final Builder builder, final int steps, final int stepId) {
        applicationFrame = new ApplicationFrame(steps, stepId);
        applicationFrame.addActionListener(this);
        this.builder = builder;
        setPanelDescriptorIdentifier(IDENTIFIER);
        setPanelComponent(applicationFrame);
    }

    /**
     * @return builder.
     */
    public Builder getBuilder() {
        return builder;
    }

    /**
     * @return the next panel.
     */
    @Override
    public Object getNextPanelDescriptor() {
        getWizard().getConfigButton().setVisible(false);
        if (applicationFrame.getApplicationsItems().size() > 0) {
            String listApplications = "";
            for (String appliItem : applicationFrame.getApplicationsItems()) {
                if (listApplications.equals("")) {
                    listApplications = appliItem;
                } else {
                    listApplications += ";" + appliItem;
                }
            }
            builder.setApplicationLocation(listApplications);
        }
        return GettingJonasOptionFrameDescriptor.IDENTIFIER;
    }

    /**
     * @return the back panel.
     */
    @Override
    public Object getBackPanelDescriptor() {
        getWizard().getConfigButton().setVisible(false);
        return LicenseFrameDescriptor.IDENTIFIER;
    }

    /**
     * Check if the next button must be enabled.
     */
    @Override
    public void aboutToDisplayPanel() {
        setNextButtonAccordingToCheckBox();
    }

    /**
     * Enable the next button according to application content list.
     */
    private void setNextButtonAccordingToCheckBox() {
        if (applicationFrame.getApplicationsItems().size() > 0) {
            getWizard().setNextFinishButtonEnabled(true);
        } else {
            getWizard().setNextFinishButtonEnabled(false);
        }
    }

    /**
     * The action performed listener.
     * @param event the event to performed.
     */
    public void actionPerformed(final ActionEvent event) {
        if (event.getSource() == applicationFrame.getJbBrowseAppli()) {
            browseApplication();
            addApplication();
            setNextButtonAccordingToCheckBox();
        } else if (event.getSource() == applicationFrame.getJbAdd()) {
            addApplication();
            setNextButtonAccordingToCheckBox();
        } else if (event.getSource() == applicationFrame.getJbDelete()) {
            deleteApplication();
            setNextButtonAccordingToCheckBox();
        } else if (event.getSource() == applicationFrame.getPathRadioButton()) {
            applicationFrame.getErrorLabel().setText("");
            applicationFrame.getApplicationPath().setEditable(true);
            applicationFrame.getJbBrowseAppli().setEnabled(true);
            applicationFrame.getApplicationUrl().setEditable(false);
        } else if (event.getSource() == applicationFrame.getUrlRadioButton()) {
            applicationFrame.getErrorLabel().setText("");
            applicationFrame.getApplicationPath().setEditable(false);
            applicationFrame.getJbBrowseAppli().setEnabled(false);
            applicationFrame.getApplicationUrl().setEditable(true);
        }
    }

    /**
     * Browse application folder.
     */
    private void browseApplication() {

        CustomFileFilter fileFilter = new CustomFileFilter(new String[] {"ear", "jar", "war", "xml"},
                "Java EE applications (.ear, .jar, .war, .xml)");
        JFileChooser fc = new JFileChooser();
        fc.setFileFilter(fileFilter);
        fc.setAcceptAllFileFilterUsed(false);
        fc.setMultiSelectionEnabled(true);
        if (fc.showOpenDialog(applicationFrame.getApplicationPath()) == JFileChooser.APPROVE_OPTION) {
            String filesName = "";
            File[] files = fc.getSelectedFiles();
            for (File file : files) {
                filesName = filesName + file.getAbsoluteFile() + ";";
            }
            filesName = filesName.substring(0, filesName.length() - 1);
            applicationFrame.getApplicationPath().setText(filesName);
        }
    }

    /**
     * add application to application list.
     */
    private void addApplication() {
        String application = "";
        if (applicationFrame.getPathRadioButton().isSelected()) {
            if (!applicationFrame.getApplicationPath().getText().equals("")) {
                application = applicationFrame.getApplicationPath().getText();
                String invalidApplic = "";
                for (String appli : Utility.parseStrToArray(application, ";")) {
                    if (Utility.checkApplication(appli)) {
                        addApplicationToList(appli);
                        applicationFrame.getApplicationPath().setText("");
                    } else {
                        invalidApplic += appli + "; ";
                    }
                }
                if (!invalidApplic.equals("")) {
                    applicationFrame.getErrorLabel().setText("Error: Invalid" + " JEE" + " application(s): " + invalidApplic);
                }

            } else {
                applicationFrame.getErrorLabel().setText("Please set the application path field !");
            }
        } else if (applicationFrame.getUrlRadioButton().isSelected()) {
            if (!applicationFrame.getApplicationUrl().getText().equals("")) {
                application = applicationFrame.getApplicationUrl().getText();
                String invalidApplic = "";
                for (String appli : Utility.parseStrToArray(application, ";")) {
                    try {
                        if (Utility.testConnexionURL(new URL(appli))) {
                            addApplicationToList(appli);
                            applicationFrame.getApplicationUrl().setText("");
                        } else {
                            invalidApplic += appli + "; ";
                        }
                    } catch (MalformedURLException ex) {
                        Logger.getLogger(ApplicationFrameDescriptor.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
                if (!invalidApplic.equals("")) {
                    applicationFrame.getErrorLabel().setText(
                            "Error : " + "Invalid URL or JEE " + " application(s): " + invalidApplic);
                }
            } else {
                applicationFrame.getErrorLabel().setText("Please set the application URL field !");
            }
        }
    }

    /**
     * Add application to the list.
     * @param application the application to add.
     */
    private void addApplicationToList(final String application) {
        if (!applicationFrame.getApplicationsItems().contains(application)) {
            applicationFrame.getApplicationsItems().add(application);
        }
        applicationFrame.getApplicationList().setListData(applicationFrame.getApplicationsItems());
    }

    /**
     * delete application from application list.
     */
    private void deleteApplication() {
        if (applicationFrame.getApplicationList().getSelectedIndices().length == 0) {
            applicationFrame.getErrorLabel().setText("Please select" + " an item !");
        } else {
            for (Object item : applicationFrame.getApplicationList().getSelectedValues()) {
                applicationFrame.getApplicationsItems().removeElement(item);
                applicationFrame.getApplicationList().setListData(applicationFrame.getApplicationsItems());
            }
        }
    }
}
