/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: BuildSummaryFrameDescriptor.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.guibuilder;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;

import javax.swing.JFileChooser;

import org.ow2.jonas.autostart.builder.Builder;
import org.ow2.jonas.autostart.frame.BuildSummaryFrame;
import org.ow2.jonas.autostart.fullwizard.FullWizardPanelDescriptor;
import org.ow2.jonas.autostart.utility.Utility;

/**
 * This class instanciates BuildSummaryFrame.
 * @author skabore
 */
public class BuildSummaryFrameDescriptor extends FullWizardPanelDescriptor implements ActionListener, KeyListener {

    /**
     * The identifier.
     */
    public static final String IDENTIFIER = "BUILD_SUMMARY_PANEL";

    /**
     * the buildSummaryFrame.
     */
    private BuildSummaryFrame buildSummaryFrame;

    /**
     * The builder librairy.
     */
    private Builder builder;

    /**
     * The constructeur.
     * @param builder the builder librairy
     * @param steps, the required steps to create a starter
     */
    public BuildSummaryFrameDescriptor(final Builder builder, final int steps) {
        this.builder = builder;
        String starterLocation = builder.getStarterJarFileLocation().getAbsolutePath();
        String starterName = builder.getStarterJarFileName();
        buildSummaryFrame = new BuildSummaryFrame(starterLocation, starterName, steps);
        buildSummaryFrame.addActionListener(this);
        buildSummaryFrame.addKeyListener(this);
        setPanelDescriptorIdentifier(IDENTIFIER);
        setPanelComponent(buildSummaryFrame);
    }

    /**
     * @return builder.
     */
    public Builder getBuilder() {
        return builder;
    }

    /**
     * @return the next panel identifier.
     */
    @Override
    public Object getNextPanelDescriptor() {
        getWizard().getConfigButton().setVisible(false);
        builder.setUseJonasFullStarter(buildSummaryFrame.getUseJonasFullStarter().isSelected());
        return BuilderFrameDescriptor.IDENTIFIER;
    }

    /**
     * @return the back panel identifier.
     */
    @Override
    public Object getBackPanelDescriptor() {
        getWizard().getConfigButton().setVisible(false);
        if (getWizard().getCurrentFrame() == 0) {
            return GettingJonasFieldFromWebsiteFrameDescriptor.IDENTIFIER;
        } else if (getWizard().getCurrentFrame() == 1) {
            return GettingJonasFieldFromDiskFrameDescriptor.IDENTIFIER;
        } else if (getWizard().getCurrentFrame() == 2) {
            return GettingJonasFieldFromZipFileFrameDescriptor.IDENTIFIER;
        }
        return GettingJonasFieldFromWebsiteFrameDescriptor.IDENTIFIER;
    }

    /**
     * Check if the next button must be enabled.
     */
    @Override
    public void aboutToDisplayPanel() {
        setNextButtonAccordingToStarterField();
    }

    /**
     * Set the next button enabled according to fields values.
     */
    private void setNextButtonAccordingToStarterField() {
        String starterName = buildSummaryFrame.getStarterName().getText();
        String starterLocation = buildSummaryFrame.getStarterLocation().getText();
        if (starterName.equals("") || starterLocation.equals("")) {
            getWizard().setNextFinishButtonEnabled(false);
            buildSummaryFrame.getErrorLabel().setText("Empty field is " + "not authorized !");
        } else if (!new File(starterLocation).canWrite()) {
            buildSummaryFrame.getErrorLabel().setText(
                    starterLocation + " can not be written!" + " Please choose other location");
            getWizard().setNextFinishButtonEnabled(false);
        } else if (!new File(starterLocation).exists()) {
            buildSummaryFrame.getErrorLabel().setText("Please set" + " a valid location !");
            getWizard().setNextFinishButtonEnabled(false);
        }/*
          * else if (!starterName.endsWith(".jar")) {
          * buildSummaryFrame.getErrorLabel().setText("Please set" +
          * " a valid name (ex.: foo.jar) !");
          * getWizard().setNextFinishButtonEnabled(false); }
          */else {
            builder.setStarterJarFileName(starterName);
            builder.setStarterJarFileLocation(new File(starterLocation));
            getWizard().setNextFinishButtonEnabled(true);
            buildSummaryFrame.getErrorLabel().setText("");
        }
        if (Utility.containFile(new File(starterLocation), starterName)) {
            buildSummaryFrame.getErrorLabel().setText(
                    starterName + " already exists in " + starterLocation + ". If you continue, it will be overwritten.");
        }
        getWizard().getConfigButton().setVisible(false);
    }

    /**
     * The action performed listener.
     * @param event the event to performed.
     */
    public void actionPerformed(final ActionEvent event) {
        if (event.getSource() == buildSummaryFrame.getJbBrowseStarterLocation()) {
            browseStarterLocation();
        } else if (event.getSource() == buildSummaryFrame.getUseJonasFullStarter()) {
            builder.setUseJonasFullStarter(buildSummaryFrame.getUseJonasFullStarter().isSelected());
        }
    }

    /**
     * Browse starter location folder.
     */
    private void browseStarterLocation() {
        JFileChooser fc = new JFileChooser();
        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fc.setAcceptAllFileFilterUsed(false);
        if (fc.showOpenDialog(buildSummaryFrame.getStarterLocation()) == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            buildSummaryFrame.getStarterLocation().setText(file.getAbsolutePath());
            setNextButtonAccordingToStarterField();
            setStarterPath();
        }
    }

    /**
     * Set the starter jar file path.
     */
    private void setStarterPath() {
        String starterName = buildSummaryFrame.getStarterName().getText();
        String starterLocation = buildSummaryFrame.getStarterLocation().getText();
        String path = starterLocation + System.getProperty("file.separator") + starterName;
        buildSummaryFrame.getStarterPath().setText(path);
    }

    /**
     * Action to do when a key is typed.
     * @param ke the key typed.
     */
    public void keyTyped(final KeyEvent ke) {
    }

    /**
     * Action to do when a key is pressed.
     * @param ke the key pressed.
     */
    public void keyPressed(final KeyEvent ke) {
    }

    /**
     * Action to do when a key is released.
     * @param ke the key released.
     */
    public void keyReleased(final KeyEvent ke) {
        if (ke.getSource() == buildSummaryFrame.getStarterLocation()) {
            setNextButtonAccordingToStarterField();
        } else if (ke.getSource() == buildSummaryFrame.getStarterName()) {
            setNextButtonAccordingToStarterField();
        }
        setStarterPath();
    }
}
