/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: BuilderFrameDescriptor.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.guibuilder;

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

import org.ow2.jonas.autostart.builder.Builder;
import org.ow2.jonas.autostart.builder.BuilderException;
import org.ow2.jonas.autostart.configuration.Configuration;
import org.ow2.jonas.autostart.frame.BuilderFrame;
import org.ow2.jonas.autostart.fullwizard.FullWizardPanelDescriptor;
import org.ow2.jonas.autostart.utility.Output;
import org.ow2.jonas.autostart.utility.Utility;

/**
 * This class instantiates BuilderFrame.
 * @author skabore
 */
public class BuilderFrameDescriptor extends FullWizardPanelDescriptor implements ActionListener {

    /**
     * The identifier.
     */
    public static final String IDENTIFIER = "BUILDER_PROCESS_PANEL";

    /**
     * The builder librairy.
     */
    private final Builder builder;

    /**
     * The build status.
     */
    private boolean buildStatus = false;

    /**
     * The builder frmae.
     */
    private final BuilderFrame builderFrame;

    /**
     * The constructeur.
     * @param builder the builder librairy
     * @param steps, the required steps to create a starter
     */
    public BuilderFrameDescriptor(final Builder builder, final int steps) {
        this.builder = builder;
        builderFrame = new BuilderFrame(steps);
        this.builder.setOutput(new Output(builderFrame.getOutputTextArea()));
        builderFrame.addActionListener(this);
        setPanelDescriptorIdentifier(IDENTIFIER);
        setPanelComponent(builderFrame);

    }

    /**
     * @return builder.
     */
    public Builder getBuilder() {
        return builder;
    }

    /**
     * @return the next panel.
     */
    @Override
    public Object getNextPanelDescriptor() {
        getWizard().getConfigButton().setVisible(false);
        getWizard().getNextButton().setEnabled(false);
        return FINISH;
    }

    /**
     * @return the back panel.
     */
    @Override
    public Object getBackPanelDescriptor() {
        getWizard().getConfigButton().setVisible(false);
        getWizard().getPreferencesButton().setEnabled(true);
        if (buildStatus) {
            return ApplicationFrameDescriptor.IDENTIFIER;
        }
        return GettingJonasOptionFrameDescriptor.IDENTIFIER;
    }

    /**
     * Check if the next button must be enabled.
     */
    @Override
    public void aboutToDisplayPanel() {
        getWizard().setNextFinishButtonEnabled(false);
        getWizard().setBackButtonEnabled(false);
        getWizard().getPreferencesButton().setEnabled(false);
    }

    /**
     * Launch the build process.
     */
    @Override
    public void displayingPanel() {
        builder.setOutput(new Output(null, builderFrame.getOutputTextArea()));
        new Thread() {
            @Override
            public void run() {
                Cursor cursor = new Cursor(Cursor.WAIT_CURSOR);
                builderFrame.setCursor(cursor);

                builderFrame.getProgressDescription().setText("");
                builderFrame.getOutputTextArea().setText("");
                builderFrame.setProgressValue(10);
                builderFrame.setProgressText("Please wait while JOnAS Builder" + " creates JOnAS server zip file !");
                // builder.setVerboseMode(true);
                builder.setVerboseMode(false);
                builder.createTmpWorkDirectory();
                builder.setConfiguration(getWizard().getConfiguration());
                try {
                    builder.createJonas();
                } catch (BuilderException e) {
                    builder.getOutput().write("Error :" + e.getMessage());
                    return;
                }
                builderFrame.setProgressValue(50);

                try {
                    builder.setStarter();
                } catch (BuilderException e) {
                    builder.getOutput().write("Error :" + e.getMessage());
                    return;
                }

                builderFrame.setProgressValue(75);

                try {
                    builder.unzipStarter();
                } catch (BuilderException e) {
                    builder.getOutput().write("Error :" + e.getMessage());
                    return;
                }
                builder.getOutput().write("Copying JOnAS server to" + " JOnAS Starter folder...");
                builderFrame.setProgressText("Copying JOnAS server to" + " JOnAS Starter folder...");
                builderFrame.setProgressValue(85);
                builder.injectJonasIntoStarter();
                builder.getOutput().write(
                        "Moving " + builder.getStarterJarFileName() + " to " + builder.getStarterJarFileLocation() + "...");
                builderFrame.setProgressText("Moving " + builder.getStarterJarFileName() + " to "
                        + builder.getStarterJarFileLocation() + "...");
                builderFrame.setProgressValue(90);
                builder.zipStarter();
                builderFrame.setProgressText("Build process successfully" + " completed. Location: "
                        + builder.getStarterDefaultLocation().getAbsolutePath());
                buildStatus = true;
                builderFrame.setProgressValue(100);

                getWizard().setNextFinishButtonEnabled(true);
                getWizard().setBackButtonEnabled(true);
                getWizard().setPreferencesButtonEnabled(true);
                cursor = new Cursor(Cursor.DEFAULT_CURSOR);
                builderFrame.setCursor(cursor);
                Utility.deleteDirectory(builder.getWorkdirectory());
                builder.setJonasRoot(null);
                builder.setJonasBase(null);
                builder.setJonasLocation(new Vector<String>());
                builder.setCreateNewJonasBase(false);
                builder.setConfiguration(new Configuration());
                Date date = new Date();
                DateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
                String jonasStarterId = dateFormat.format(date).toString();
                builder.setStarterJarFileName("jonas-starter" + jonasStarterId + ".jar");
                builder.setStarterJarFileLocation(new File(System.getProperty("user.dir")));
            }
        }.start();
    }

    /**
     * The action performed listener.
     * @param event the event to performed.
     */
    public void actionPerformed(final ActionEvent event) {
        if (event.getSource() == builderFrame.getShowDetailButton()) {
            builderFrame.showDetail();
        }
    }
}
