/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: GuiBuilder.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.guibuilder;

import java.net.MalformedURLException;
import java.net.URL;

import org.ow2.jonas.autostart.builder.Builder;
import org.ow2.jonas.autostart.configuration.Configuration;
import org.ow2.jonas.autostart.configurationframe.ConfiguratorFrameDescriptor;
import org.ow2.jonas.autostart.fullwizard.FullWizard;
import org.ow2.jonas.autostart.fullwizard.FullWizardPanelDescriptor;
import org.ow2.jonas.autostart.utility.Utility;

/**
 * This class lauches the JOnAS AutoStart Builder tool.
 * @author skabore
 */
public final class GuiBuilder {

    /**
     * Google url.
     */
    private static final String GOOGLE_URL = "http://www.google.com";

    /**
     * The builder librairy.
     */
    private Builder builder;

    /**
     * Setps to create a starter jar file.
     */
    private final static int steps = 8;

    /**
     * Setp identifier.
     */
    private final static int stepId = 6;

    /**
     * Default constructor.
     */
    private GuiBuilder() {

        Configuration configuration = new Configuration();
        configuration.initialize();

        builder = new Builder();
        Utility.setProxyValues(builder.getAutoStartProxy());
        builder.setAutoStartProxy(Utility.getAutoStartProxy());

        builder.setConfiguration(configuration);

        FullWizard wizard = new FullWizard();
        wizard.getDialog().setTitle("JOnAS AutoStart :: Builder");
        wizard.getPreferencesButton().setVisible(true);
        wizard.setBuilder(builder);

        FullWizardPanelDescriptor descriptor1 = new FirstFrameDescriptor(steps);
        wizard.registerWizardPanel(FirstFrameDescriptor.IDENTIFIER, descriptor1);

        FullWizardPanelDescriptor descriptor4 = new LicenseFrameDescriptor(steps);
        wizard.registerWizardPanel(LicenseFrameDescriptor.IDENTIFIER, descriptor4);

        FullWizardPanelDescriptor descriptor2 = new ApplicationFrameDescriptor(builder, steps, 3);
        wizard.registerWizardPanel(ApplicationFrameDescriptor.IDENTIFIER, descriptor2);

        FullWizardPanelDescriptor descriptor3 = new GettingJonasOptionFrameDescriptor(steps);
        wizard.registerWizardPanel(GettingJonasOptionFrameDescriptor.IDENTIFIER, descriptor3);

        FullWizardPanelDescriptor descriptor6 = new BuilderFrameDescriptor(builder, steps);
        wizard.registerWizardPanel(BuilderFrameDescriptor.IDENTIFIER, descriptor6);

        FullWizardPanelDescriptor descriptor8 = new GettingJonasFieldFromWebsiteFrameDescriptor(builder, steps);
        wizard.registerWizardPanel(GettingJonasFieldFromWebsiteFrameDescriptor.IDENTIFIER, descriptor8);

        FullWizardPanelDescriptor descriptor9 = new GettingJonasFieldFromDiskFrameDescriptor(builder, steps);
        wizard.registerWizardPanel(GettingJonasFieldFromDiskFrameDescriptor.IDENTIFIER, descriptor9);

        FullWizardPanelDescriptor descriptor10 = new GettingJonasFieldFromZipFileFrameDescriptor(builder, steps);
        wizard.registerWizardPanel(GettingJonasFieldFromZipFileFrameDescriptor.IDENTIFIER, descriptor10);

        FullWizardPanelDescriptor descriptor11 = new BuildSummaryFrameDescriptor(builder, steps);
        wizard.registerWizardPanel(BuildSummaryFrameDescriptor.IDENTIFIER, descriptor11);

        ConfiguratorFrameDescriptor configuratorFrameDescriptor = new ConfiguratorFrameDescriptor(builder.getConfiguration(),
                stepId, steps);
        configuratorFrameDescriptor.setBuildSummaryFrameId(BuildSummaryFrameDescriptor.IDENTIFIER);
        configuratorFrameDescriptor.setGettingJonasFieldFromDiskFrameId(GettingJonasFieldFromDiskFrameDescriptor.IDENTIFIER);
        configuratorFrameDescriptor
                .setGettingJonasFieldFromWebsiteFrameId(GettingJonasFieldFromWebsiteFrameDescriptor.IDENTIFIER);
        configuratorFrameDescriptor
                .setGettingJonasFieldFromZipFileFrameId(GettingJonasFieldFromZipFileFrameDescriptor.IDENTIFIER);

        FullWizardPanelDescriptor descriptor7 = configuratorFrameDescriptor;
        wizard.registerWizardPanel(ConfiguratorFrameDescriptor.IDENTIFIER, descriptor7);

        wizard.setCurrentPanel(FirstFrameDescriptor.IDENTIFIER);
        wizard.showModalDialog();
        try {
            if (!Utility.testConnexionURL(new URL(GOOGLE_URL))) {
                wizard.showMessage("Can not established the Internet" + " connection.\n"
                        + "If you have a proxy, set the proxy parameters\n" + " using the 'Preferences' button.");
            }
        } catch (MalformedURLException ex) {
            // Logger.getLogger(this.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * The main function.
     * @param args the program arguments
     */
    public static void main(final String[] args) {
        new GuiBuilder();
    }
}
